unit UnitFrmScratchpad;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, UnitKeyboardQuery, StdCtrls, Buttons;

type
  TFrmScratchPad = class(TForm)
    imgScratch: TImage;
    TimCloser: TTimer;
    pnlTop: TPanel;
    StaticText1: TStaticText;
    Shape1: TShape;
    btnHelp: TSpeedButton;
    btnGestures: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormMouseEnter(Sender: TObject);
    procedure FormMouseLeave(Sender: TObject);
    procedure TimCloserTimer(Sender: TObject);
    procedure imgScratchMouseEnter(Sender: TObject);
    procedure imgScratchMouseLeave(Sender: TObject);
    procedure pnlTopMouseEnter(Sender: TObject);
    procedure pnlTopMouseLeave(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure btnHelpMouseEnter(Sender: TObject);
    procedure btnHelpMouseLeave(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure btnGesturesClick(Sender: TObject);
  private
    { Private declarations }
    storedWidth : integer;
    p : TPanel;
    FirstDown : boolean;
    showingfull : boolean;
    ForceHide : boolean;
    forwin : THandle;
    procedure HidePad;
    procedure ShowPad(nowidth : boolean = false);
    procedure SetWidth(w : integer; fudge : integer = 0);
    function GetInterval: integer;
    procedure SetInverval(const Value: integer);
    procedure SetEnable(const Value: boolean);
  protected
    procedure WMNCHitTest(VAR Msg: TWMNcHitTest); { message WM_NCHITTEST;}
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public declarations }
    procedure ReportPoint(p : TPoint);
    procedure ReportMouseButton(down : boolean);
    function RemoveSelfFromFocus(NoFocus : boolean = false) : THandle;

    property prFadeTime : integer read GetInterval write SetInverval;
    property prEnable : boolean write SetEnable;


  end;

var
  FrmScratchPad: TFrmScratchPad;

implementation

uses UnitMisc, UnitFrmGestures, UnitFrmSysTrayMenu, UnitFrmAbout, UnitFrmConfig,
  UnitFrmMouseMonitor;


{$R *.dfm}
procedure TFrmScratchPad.btnGesturesClick(Sender: TObject);
begin
    self.HidePad;
    self.TimCloser.Enabled := false;


    if ThreadAttach(GetForegroundWindow) then begin
        FrmGestures.Show;
        //SetForegroundWindow(frmGestures.handle);
        ThreadDetach;
    end;

end;

procedure TFrmScratchPad.btnHelpClick(Sender: TObject);
begin
    self.HidePad;
    self.TimCloser.Enabled := false;
    // we're focus and can set the foreground
    FrmConfig.ShowPage(2);
end;

procedure TFrmScratchPad.btnHelpMouseEnter(Sender: TObject);
begin
    FormMouseEnter(sender); 
end;

procedure TFrmScratchPad.btnHelpMouseLeave(Sender: TObject);
begin
    FormMouseLeave(sender); 
end;

procedure TfrmScratchpad.CreateParams(var Params: TCreateParams);
begin
   inherited CreateParams(Params);

    // styles copied from HintWindow class, some may do nothing
    with Params do begin
        WindowClass.style := WindowClass.style or CS_DROPSHADOW or CS_SAVEBITS;
        style := style or WS_POPUP {or WS_BORDER };
        ExStyle := ExStyle  or WS_EX_TOOLWINDOW;
        WndParent := GetDesktopwindow;
    end;
end;

procedure TFrmScratchPad.FormActivate(Sender: TObject);
begin
    UnitMisc.RemoveFocusFromSelf;
end;

procedure TFrmScratchPad.FormCreate(Sender: TObject);
begin
    storedWidth := self.Width;


    ClientHeight := Height;
    ClientWidth := Width;


    imgScratch.Canvas.Brush.Color := clWhite;
    imgScratch.Canvas.Pen.Color := clWhite;


    // (dirty, filth tricks because nothing worked to hide just this window)
    // Hide the entire application from the taskbar, remove the caption and border
    // Only hidden windows are affected
    // Any form needing to show on the taskbar must manually toggle WS_EX_TOOLWINDOW off and WS_EX_APPWINDOW on
    ShowWindow(Application.Handle, SW_HIDE);
    SetWindowLong(Application.Handle, GWL_EXSTYLE,
        GetWindowLong(Application.Handle, GWL_EXSTYLE) or WS_EX_TOOLWINDOW);
    {SetWindowLong(self.Handle, GWL_EXSTYLE,
        GetWindowLong(self.Handle, GWL_EXSTYLE) or WS_EX_TOOLWINDOW);}
    //ShowWindow(FrmGestures.Handle,SW_HIDE);

    
    pnlTop.Align := alTop;
    imgScratch.Align := alClient;
    imgScratch.Width := width;

    FirstDown := true;
    self.showingfull := true;


    self.ShowPad(true); 
    self.HidePad;

end;



procedure TFrmScratchPad.FormMouseEnter(Sender: TObject);
begin
    forwin := Windows.GetForegroundWindow;
    forwin := self.RemoveSelfFromFocus(true);

    self.ShowPad;
    if not ForceHide  then begin
        self.timcloser.enabled := false;
    end;
end;

procedure TFrmScratchPad.FormMouseLeave(Sender: TObject);
begin
    if self.showingfull then
        TimCloser.Enabled := true;
end;


procedure TFrmScratchPad.ShowPad(nowidth : boolean = false);
var m : TMonitor;
begin
    forwin := self.RemoveSelfFromFocus(true);

    SetWindowLong(self.Handle, GWL_EXSTYLE,
        GetWindowLong(self.Handle, GWL_EXSTYLE) or WS_EX_TOOLWINDOW and not WS_EX_APPWINDOW);

    self.showingfull := true;
    //SetWindowPos(handle, HWND_TOPMOST, self.left, self.top, self.Width, self.height,SWP_NOMOVE or SWP_NOSIZE  );

    SetWindowPos(handle, HWND_NOTOPMOST, self.left, self.top, self.Width, self.height,SWP_NOMOVE or SWP_NOSIZE  );

    m := Screen.PrimaryMonitor;
    self.Top := m.Top + trunc(m.Height/2) - trunc(self.Height/2);
    if not nowidth then
        SetWidth(storedWidth, 3);
    {
    if ThreadAttach(GetForegroundWindow)  then begin
        Windows.SetForegroundWindow(self.handle);
        //self.Show;
        ThreadDetach;
    end;
    //}
end;

procedure TFrmScratchPad.HidePad;
var m : TMonitor;
begin
    self.TimCloser.Enabled := false;
    //self.Show;
    SetWindowPos(handle, HWND_TOPMOST, self.left, self.top, 0, 0, SWP_NOSIZE or SWP_NOMOVE );

    m := Screen.PrimaryMonitor;
    //self.Top := m.Top + trunc(m.height);
    self.Top := m.Top + trunc(m.Height/2) - trunc(self.Height/2);
    SetWidth(5);
    self.showingfull := false;
    imgScratch.Canvas.Pen.Color := clWhite;
    imgScratch.Canvas.FillRect(imgScratch.Canvas.ClipRect);

    //self.RemoveSelfFromFocus; 
end;

procedure TFrmScratchPad.SetWidth(w: integer; fudge : integer);
var m : TMonitor;
begin
    self.Width := w;
    m := Screen.PrimaryMonitor;
    //self.Top := 0;
    //self.Top := m.Top + trunc(m.Height/2) - trunc(self.Height/2);
    self.Left := m.Left+m.width - (self.Width + fudge);
end;





procedure TFrmScratchPad.imgScratchMouseEnter(Sender: TObject);
begin
    self.FormMouseEnter(self);
end;
procedure TFrmScratchPad.imgScratchMouseLeave(Sender: TObject);
begin
    self.FormMouseLeave(sender);
end;
procedure TFrmScratchPad.pnlTopMouseEnter(Sender: TObject);
begin
    self.FormMouseEnter(sender);
end;

procedure TFrmScratchPad.pnlTopMouseLeave(Sender: TObject);
begin
    self.FormMouseLeave(sender);
end;

function TFrmScratchPad.RemoveSelfFromFocus(NoFocus : boolean) : thandle;
var h : THandle;
    name : string;
begin

    //SetWindowPos(handle, HWND_NOTOPMOST, self.left, self.top, self.Width, self.height,SWP_NOMOVE or SWP_NOSIZE  );

    h := GetForegroundWindow;

    name := WindowHandleToEXEName(h);
    if lowercase(name) = 'mousewrangler.exe' then begin
        h := UnitMisc.GetNextWindow(h);
        name := WindowHandleToEXEName(h);

        if lowercase(name) = 'mousewrangler.exe' then begin
            h := UnitMisc.GetNextWindow(h);
            name := WindowHandleToEXEName(h);

            if lowercase(name) = 'mousewrangler.exe' then begin
                h := UnitMisc.GetNextWindow(h);
                name := WindowHandleToEXEName(h);
            end;
        end;
    end;

        
    //FrmMouseMonitor.AppendLog('for - ' + name);
    //name := WindowHandleToEXEName(GetForegroundWindow);
    //FrmMouseMonitor.AppendLog('for - ' + WindowHandleToEXEName(GetForegroundWindow)  );


    if not NoFocus then
        if lowercase(name) = 'mousewrangler.exe' then begin
            if ThreadAttach(h) then begin

                Windows.SetForegroundWindow(h);
                ThreadDetach;
            end;
        end else begin
            if ThreadAttach(GetForegroundWindow) then begin
                Windows.SetForegroundWindow(h);
                ThreadDetach;
            end;
        end;

    result := h;
end;

procedure TFrmScratchPad.ReportMouseButton(down : boolean);
var h : THandle;
    name : string;
begin
    if self.showingfull  then begin
        if down then begin
            if FirstDown then begin
                imgScratch.Canvas.Pen.Color := clWhite;
                imgScratch.Canvas.FillRect(imgScratch.Canvas.ClipRect);
                imgScratch.Canvas.Pen.Color := clBlack;
                ReportPoint(mouse.cursorpos);
                FirstDown := false;
            end else begin
                //ReportPoint(mouse.cursorpos);
            end;
        end else begin
            if not firstdown then begin
                ForceHide := true;
            end;

            FirstDown := true;
        end;
    end else begin
        FirstDown := true;
    end;
end;

procedure TFrmScratchPad.ReportPoint(p: TPoint);
begin
    imgScratch.Canvas.Pen.Color := clBlack;
    p := imgScratch.ScreenToClient(p);
    if firstdown then begin
        imgScratch.Canvas.MoveTo(p.X,p.y);
    end else begin
        imgScratch.Canvas.LineTo(p.X,p.y);
    end;
end;

function TFrmScratchPad.GetInterval: integer;
begin
    result := TimCloser.Interval;
end;

procedure TFrmScratchPad.SetEnable(const Value: boolean);
begin
    if not value then
        self.Hide
    else begin
        self.show;
        self.HidePad;
    end;
    
end;

procedure TFrmScratchPad.SetInverval(const Value: integer);
begin
    TimCloser.Interval := value;
end;



procedure TFrmScratchPad.TimCloserTimer(Sender: TObject);
begin
    TimCloser.Enabled := false;
    self.ForceHide := false;
    self.HidePad;
end;

procedure TFrmScratchPad.WMNCHitTest(var Msg: TWMNcHitTest);
var r : TRect;
begin
    msg.result := HTTRANSPARENT ;

    windows.GetWindowRect(self.Handle, r);
    if Windows.PtInRect(r, Point(msg.XPos, msg.YPos)  ) then begin
        if not self.ForceHide  then begin
            forwin := Self.RemoveSelfFromFocus(true);
            self.ShowPad;

            TimCloser.Enabled := false;
            //TimCloser.Enabled := true;
        end;
    end else begin

    end;
end;

end.



